'*******************************************************************************
'        PRIMJER PROGRAMA ZA AUTOKALIBRACIJU IZABRANOG ADC ULAZA

'Mikrokontroler: ATtiny44 ili ATtiny84

'Kalibracija se provodi u dvije toke (minimum i maksimum).

'Koristi naredbu Adc_correction iz programske knjiice "Adcx.inc"

'za korekciju uitanog analognog podatka iz Word varijable Adc_input,prema

'kalibracijskim parametrima; rezultat korekcije je u Single varijabli Adcx.

'Varijable Adc_input i Adcx dimenzionirane su unutar knjiice "Adcx.inc".

'Nakon kalibracije i nakon izvrene naredbe Adc_correction:

' Naponski signal < Minimum --> Adcx=0 , Under_min=1 , Over_max=0

' Naponski signal > Maksimum --> Adcx=1023 , Under_min=1 , Over_max=1

' Minimum <= Naponski signal <= Maksimum --> Adcx=0..1023, Under_min=Over_max=0

'Kalibracijski parametri Offset i Gain  i bajt za potvrdu izvrenja kalibracije

'uvaju se u internom EEPROM-u od adrese 1 do adrese 7 , adresu 0 ne koristiti.

'Ako kalibracija nije napravljena program ju automatski starta nakon reseta.

'Ako se kalibracija hoe ponoviti, nakon reseta dri tipku Calibration_key 5s.
'-------------------------------------------------------------------------------
'Analogni signal,npr.sa potenciometra kojim se simulira temperaturni pretvornik,
'dovodi se na ADC0 ulaz.

'ADC0 ulaz knjiicom "Adcx.inc" treba kalibrirati u dvije toke za mjerni opseg
'kalibracije M.O.K. -20C do 100C:

'1.toka (Minimum):-20C,namjesti napon koji odgovara temperaturi -20C,npr.1V
'2.toka (Maximum):100C,namjesti napon koji odgovara temperaturi 100C,npr.4V

'Autor: Mladen empuh , Datum 11.06.2023. e-mail: cempuh.mladen@gmail.com
'*******************************************************************************

'$regfile = "attiny84.dat"                                   ' za ATtiny84
$regfile = "attiny44.dat"                                   ' za ATtiny44
$crystal = 8000000                                          ' 8MHz
'$crystal = 1000000                                          ' 1MHz
$swstack = 50
$hwstack = 40
$framesize = 64

'Napomena:
'Nakon prve kalibracije fuse bitovima mikrokontrolera onemogui brisanje sadraja
'internog EEPROM-a kako bi kalibracijski parametri bili sauvani kod novog upisa programa.
'------------------------------------------------------------
'        DEFINIRANJE PINOVA ZA LCD U 4 BITNOM RADU
'------------------------------------------------------------
Config Lcd = 16 * 2                                         ' izabran LCD 1602

Config Lcdpin = Pin , Db4 = Porta.4 , Db5 = Porta.5 , Db6 = Porta.6 , Db7 = Porta.7 , Rs = Porta.2 , E = Porta.3
Cls
Cursor Off
'------------------------------------------------------------
Lcd_backlight Alias Portb.2                                 ' opcija izbora izlaza za ukljuenje/iskljuenje rasvjete LCD-a
Config Lcd_backlight = Output
Lcd_backlight = 1                                           ' 1 = ukljui , 0 = iskljui rasvjetu LCD-a

'------------------------------------------------------------
'        INICIJALIZACIJA AD KONVERTERA
'------------------------------------------------------------
Config Adc = Single , Prescaler = Auto , Reference = Avcc   'inicijaliziraj ADC za pretvorbu od 0 do  VCC (VCC=5V)

Start Adc                                                   ' startaj ADC pretvorbu

Channel Alias 0                                             ' upii redni broj izabranog ADC ulaza za kalibraciju, izabran ADC0

'------------------------------------------------------------
'Izaberi ulaz za kalibracijsku tipku:

Calibration_key Alias Pina.1                                ' ako je nakon reseta kalibracijska tipka na ulazu Calibration_key pritisnuta 5s
                                                             ' starta kalibracija
Config Calibration_key = Input                              ' kalibracija starta i automatski ako prije nije bila izvrena

Porta.1 = 1                                                 ' ukljui interni pull up otpornik na ulazu PINA.1

'-----------------------------------------------------------

$include "Adcx.inc"                                         ' ukljui knjiicu "Adcx.inc" za autokalibraciju izabranog ADC ulaza i za naredbu Adc_correction

'-----------------------------------------------------------
Dim Temp1 As Single                                         ' Single varijabla u korisnikom programu za izraun temperature

Do

  Adc_input = Getadc(channel)                               ' konvertira napon sa ADC ulaza Channel,npr.napon sa potenciometra,
                                                             ' naziv varijable za ADC pretvorbu mora biti Adc_input

  Adc_correction                                            ' korekcija ADC iznosa sukladno kalibraciji i upis u Single varijablu Adcx
                                                             ' i postavlja Byte varijable Under_min i Over_max u stanje 0 ili 1

  Locate 1 , 1 : Lcd "ADCX=" ; Fusing(adcx , "#.###") ; "   "       ' ispis na LCD korigiranog ADC iznosa iz Single varijable Adcx

  If Under_min = 0 And Over_max = 0 Then                    ' ako je ulazni napon unutar mjernog opsega kalibracije tada iz vrijednosti Adcx
                                                            ' izraunaj i ispii temperaturu Temp1
   Temp1 = Adcx * 120
   Temp1 = Temp1 / 1023
   Temp1 = Temp1 - 20                                       ' Temp1 = Adcx*(100-(-20))/1023 + (-20) C  ; M.O.K je od -20C do 100C

   Locate 2 , 1 : Lcd "Temp1 = " ; Fusing(temp1 , "#.#") ; Chr(223) ; "C  "       ' ispis temperature Temp1

  End If

  If Under_min = 1 Then                                     ' ulazni napon je ispod mjernog opsega kalibracije
   Locate 2 , 1 : Lcd "Temp1 < -20" ; Chr(223) ; "C   "     ' ispii da je Temp1 < -20C
  End If

  If Over_max = 1 Then                                      ' ulazni napon je iznad mjernog opsega kalibracije
   Locate 2 , 1 : Lcd "Temp1 > 100" ; Chr(223) ; "C   "     ' ispii da je Temp1 > 100 C
  End If

  Waitms 100                                                ' opcija usporavanja Do-Loop petlje, a time i uestalosti AD konverzije, nije nuno!
  
Loop

'------------------------------------------------------------

End                                                         ' kraj programa