'***********************************************************************
'        Self kalibracija izabranog ulaza i naredba Adc_correction

'Naziv datoteke: Adcx.inc

'Kalibracija se provodi u dvije tocke (minimum i maksimum signala)

'Podrzava kalibraciju jednog izabranog ADC ulaza

'ADC ulaz izabran je u korisnickom programu sa Channel Alias x

'Tipka Calibration_key je za kalibraciju i za prihvat podataka

'Kalibracijski podaci (Offset i Gain) spremaju se u interni EEPROM

'Naredba Adc_correction je za korekciju iznosa ADC ulaza sukladno

'kalibraciji i upisuje rezultat korekcije u Single varijablu Adcx

'Offset je Word varijabla, pa Offset iznos moze biti i veci od 255

'Byte varijabla Under_min je 1 ako je Adc_input < Offset iznosa

'Byte varijabla Over_max je 1 ako je Adc_input > kalibracijskog maksimuma

'Autor: Mladen Cempuh , Datum 20.11.2021. cempuh.mladen@gmail.com
'************************************************************************

Config Submode = New                                        'use sub/function without Declare
'------------------------------------------------------------
'RAM varijable:

Dim Adc_input As Word                                       ' Varijabla za podatak ADC sa ulaza Channel
Dim Adc_input_max As Word                                   ' Varijabla za maksimalni podatak ADC sa ulaza Channel
Dim Next_adc_read As Word                                   ' Varijabla za N uzastopnih citanja ADC ulaza Channel za odredjivanje maksimuma Offseta
Dim Adcx As Single                                          ' Varijabla Single za podatak nakon korekcije ADC podatka Adc_input
Dim Offset As Word                                          ' Varijabla za offset tj. ADC iznos kod minimalnog ulaznog napona
Dim Gain As Single                                          ' Varijabla za korekciju pojacanja (nagib pravca)
Dim Calibrated_ok As Byte                                   ' Varijabla za potvrdu izvrenja kalibracije, 1=izvreno
Dim Temp_word As Word                                       ' Pomocna Word varijabla
Dim Over_max As Byte                                        ' Adc_correction postavlja Over_max=1 ako je Adc_input > kalibracijskog maksimuma, u protivnom je 0
Dim Under_min As Byte                                       ' Adc_correction postavlja Under_min=1 ako je Adc_input < kalibracijskog minimuma, u protivnom je 0


'EEPROM varijable internog EEPROM-a:

Dim Not_used As Eram Byte                                   ' interna EEPROM lokacija na adresi 0 ne smije se koristi (nepouzdana)
Dim Calibrated_ok_eeprom As Eram Byte                       ' ako je 1 kalibracija izvrsena
Dim Offset_eeprom As Eram Word                              ' za pohranu Offset iznosa
Dim Gain_eeprom As Eram Single                              ' za pohranu Gain iznosa                             '
'-------------------------------------------------------------

 Calibrated_ok = Calibrated_ok_eeprom                       'ucitaj u Byte varijablu Calibrated_ok iznos iz EEPROM varijable Calibrated_ok_eeprom
 If Calibrated_ok <> 1 Then                                 ' ako je Calibrated_ok <> 1 idi na kalibraciju
  Gosub Calibration
  Goto Lab_load_data                                        ' nakon kalibracije idi na Lab_load_data za ucitavanje iznosa Offest i Gain
 End If

Cls : Lcd "Calibration key"                                 'Calibrated_ok = 1 , ispisi "Calibration key 5s for calibrate"
Locate 2 , 1 : Lcd "5s for calibrate"

Lab_wait_5s:
If Calibration_key = 1 Then Goto Lab_load_data              ' ako nije pritisnuta tipka Calibration_key idi na Lab_load_data
Incr Temp_word
Waitms 100
If Temp_word < 50 Then Goto Lab_wait_5s                     ' ceka 5s pritisnutu tipku Calibration_key za ulazak u kalibraciju
Gosub Calibration                                           ' tipka Calibration_key je pritisnuta 5s, idi na ponovnu kalibraciju

Lab_load_data:
 Offset = Offset_eeprom                                     ' ucitaj Offset iznos iz EEPROM-a
 Gain = Gain_eeprom                                         ' ucitaj Gain iznos iz EEPROM-a
Cls                                                         ' obrisi LCD
Goto Lab_exit_calibration                                   ' izlaz iz datoteke "Adcx.inc"

Sub Adc_correction()                                        ' naredba Adc_correction upisuje rezultat korekcije ADC ulaza u Single varijablu Adcx
 Over_max = 0 : Under_min = 0                               ' pocetno postavi Byte varijable Over_max i Under_min u 0
 If Adc_input < Offset Then                                 ' za Adc_input < Offset je Byte varijabla Under_min=1 i Single varijabla Adcx = 0
    Under_min = 1
    Adcx = 0
    Goto Lab_exit_sub
 End If
 Temp_word = Adc_input - Offset
 Adcx = Gain * Temp_word                                    ' Adcx = Gain * (Adc_input-Offset)
 If Adcx > 1023 Then                                        ' ako je Adcx > 1023 (tada je Adc_input > kalibracijskog maksimuma)
  Over_max = 1                                              ' tada je Byte varijabla Over_max=1
  Adcx = 1023                                               ' i Single varijabla Adcx je 1023
 End If
Lab_exit_sub:
End Sub

'------------------------------------------------------------
'Potprogram za kalibraciju izabranog ADC ulaza

Calibration:

Cls : Lcd ">>Calibration<<" : Wait 3                        ' ispisi na LCD pocetak ciklusa kalibracije
Gosub Calibration_key_free                                  ' cekaj otpustanje kalibracijske tipke
Cls : Locate 2 , 1 : Lcd "Calib.key for OK"

'Prikljuci na izabrani ADC ulaz iznos mjernog signala koji odgovara minimalni iznosu mjerene velicine:1.tocka kalibracije=Offset

Lab_first_point:
Temp_word = 0                                               ' postavi pomocnu varijablu Temp_word u 0
For Next_adc_read = 1 To 400                                ' citaj 400 puta ADC ulaz Channel i pronadji maksimum za Offset
 Adc_input = Getadc(channel)                                ' ucitaj ADC sa ulaza Channel u varijablu Adc_input
 If Adc_input > Temp_word Then Temp_word = Adc_input        ' nadji maksimalnu vrijednost signala na ADC ulazu i spremi ga u Temp_word
Next Next_adc_read                                          ' slijedece citanje ADC ulaza Channel

Locate 1 , 1 : Lcd "Minimum: " ; Temp_word ; "   "          'Ispisi najveci iznos za Minimum iz varijable Temp_word
If Calibration_key = 0 Then                                 ' potvrdi pritiskom na kalibracijsku tipku
  Offset = Temp_word                                        ' to je Offset iznos
  Gosub Calibration_key_free                                ' cekaj otpustanje kalibracijske tipke
  Goto Lab_second_point                                     ' i idi na drugu tocku kalibracije
End If
Goto Lab_first_point

'Prikljuci na izabrani ADC ulaz iznos mjernog signala koji odgovara maksimalnom iznos mjerene velicine: 2. tocka kalibracije za Gain izracun

Lab_second_point:
Adc_input = Getadc(channel)                                 ' ucitaj ADC sa ulaza Channel u varijablu Adc_input
Locate 1 , 1 : Lcd "Maximum: " ; Adc_input ; "   "          ' prikljuci na izabrani ADC ulaz maksimalni iznos mjernog naponskog signala:2.toka kalibracije
                                                             ' kod kojeg zelis da nakon korekcije  naredbom Adc_correction iznos ADC pretvorbe bude 1023
If Calibration_key = 0 And Adc_input > Offset Then          ' ako je na izabrani ADC ulaz spojen naponski signal kome je ADC iznos vei od Offset iznosa i
  Temp_word = Adc_input - Offset                            ' ako je pritisnuta kalibracijska tipka, tada izracunaj iznos za Gain
  Gain = 1023 / Temp_word                                   ' Gain = 1023 / (Adc_input - Offset)  ; Adc_input je naponski mjerni signal za maksimum
  Gosub Calibration_key_free                                ' cekaj otpustanje kalibracijske tipke
  Goto Save_calibration                                     ' idi na upis Offset i Gain iznosa u interni EEPROM
End If
Goto Lab_second_point

Save_calibration:

 Offset_eeprom = Offset                                     ' upis Offset iznosa u EEPROM
 Gain_eeprom = Gain                                         ' upis Gain iznosa u EEPROM
 Calibrated_ok = 1                                          ' upisi 1 u EEPROM lokaciju Calibrated_ok_eeprom za oznaku izvrsenja kalibracije
 Calibrated_ok_eeprom = Calibrated_ok
 Cls : Lcd "  Calibrated !" : Wait 3                        'obrisi LCD i ispisi poruku "Calibrated !" za obavljenu kalibraciju
 Return                                                     ' vrati se iz potprograma kalibracije

Calibration_key_free:
Debounce Calibration_key , 1 , Key_is_free                  'potprogram za cekanje na otpustanje kalibracijske tipke
Goto Calibration_key_free
Key_is_free:
Return

Lab_exit_calibration:                                       ' kraj "Adcx.inc" datoteke